from .core import (
    get_dist, 
    get_dist_full, 
    get_dm, 
    get_dm_full, 
    get_galactic_dm
)

from .tests import test

__all__ = [
    'get_dist', 
    'get_dist_full', 
    'get_dm', 
    'get_dm_full', 
    'get_galactic_dm',
    'test',
    ]


import unittest
import os

def test():
    """ Run all unit tests """
    loader = unittest.TestLoader()
    suite = loader.discover(os.path.dirname(__file__))
    runner = unittest.TextTestRunner()
    runner.run(suite)


import unittest
from pyne2001 import *


class TestFunctions(unittest.TestCase):
    """ """
    def test_get_dm(self):
        get_dm(0, 0, 10.0)
    
    def test_get_dm_full(self):
        items = get_dm_full(0, 0, 10.0)

    def test_get_dist(self):
        get_dist(0, 0, 400.0)
    
    def test_get_dist_full(self):
        items = get_dist_full(0, 0, 400.0)

    def test_get_galactic_dm(self):
        get_galactic_dm(0, 0)

    def test_inversion1(self):
        glon = 5.0
        glat = 5.0
        dist_kpc = 1.0
        dm = get_dm(glon, glat, dist_kpc)
        dist_kpc_calculated, __ = get_dist(glon, glat, dm)
        self.assertAlmostEqual(dist_kpc, dist_kpc_calculated, places=3)

    def test_inversion2(self):
        glon = 10.0
        glat = 10.0
        dm = 100.0
        dist_kpc, __ = get_dist(glon, glat, dm)
        dm_calculated = get_dm(glon, glat, dist_kpc)
        self.assertAlmostEqual(dm, dm_calculated, places=3)


if __name__ == "__main__":
    unittest.main()



import os
import setuptools
import subprocess
from setuptools import setup
from setuptools.command.install import install
from setuptools.command.develop import develop
from setuptools.command.build_py import build_py


def compile_ne2001():
    try:
        cwd = os.getcwd()
        thisdir, __ = os.path.split(__file__)
        srcdir = os.path.realpath(os.path.join(thisdir, 'pyne2001', 'NE2001', 'src'))
        os.chdir(srcdir)
        subprocess.check_call('make clean', shell=True)
        subprocess.check_call('make all', shell=True)
    except:
        raise
    finally:
        os.chdir(cwd)


class CustomInstall(install):
    def run(self):
        self.announce("Compiling NE2001 FORTRAN code")
        compile_ne2001()
        install.run(self)


class CustomDevelop(develop):
    def run(self):
        self.announce("Compiling NE2001 FORTRAN code")
        compile_ne2001()
        develop.run(self)


# NOTE: It is necessary to also have a custom build_py command
# During the first 'pip install' on a given machine, pip caches the packages
# with all its build files into a wheel (.whl). To do so, it runs the 
# 'build_py' command and stores all resulting files in the .whl. If
# 'build_py' does not call compile_ne2001(), then the .whl will NOT contain
# the compiled binaries. And in this case, running pip install another
# time (in another conda environment for example) actually installs that
# cached .whl which contains no binaries, and the module does not work.
class CustomBuildPy(build_py):
    def run(self):
        self.announce("Compiling NE2001 FORTRAN code")
        compile_ne2001()
        build_py.run(self)


with open("README.md", "r") as fh:
    long_description = fh.read()


setup(
    name='pyne2001',
    url='https://github.com/v-morello/pyne2001',
    author='Vincent Morello',
    author_email='vmorello@gmail.com',
    description='A simple python wrapper around the original FORTRAN implementation of the NE2001 Galactic free electron density model',
    long_description=long_description,
    long_description_content_type='text/markdown',
    version='0.1.1',
    packages=setuptools.find_packages(),
    install_requires=[],
    license='MIT License',

    # NOTE (IMPORTANT): This means that everything mentioned in MANIFEST.in will be copied at install time 
    # to the package’s folder placed in 'site-packages'
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Fortran",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Unix",
        "Topic :: Scientific/Engineering :: Astronomy"
        ],
    cmdclass={
        'install': CustomInstall,
        'develop': CustomDevelop,
        'build_py': CustomBuildPy
        }
)



from __future__ import print_function
import os

try:
    # Python 3+
    from subprocess import getoutput
except ImportError:
    # Python 2
    from commands import getoutput


def _get_executable_dir():
    thisdir, __ = os.path.split(__file__)
    path = os.path.join(thisdir, "NE2001", "bin")
    return os.path.realpath(path)


executable_dir = _get_executable_dir()


def _run_ne2001(glon, glat, dmd, ndir):
    """
    dmd: float
        DM or distance in kpc, depending on ndir
    ndir: int
        If 1, convert DM to distance. If -1, convert distance to DM.
    """
    try:
        cwd = os.getcwd()
        os.chdir(executable_dir)
        out = None
        if dmd < 0:
            raise ValueError("dmd cannot be negative")
        if not ndir in (-1, 1):
            raise ValueError("ndir must be -1 or 1")

        cmd = "./NE2001 {:.6f} {:.6f} {:.6f} {:d}".format(glon, glat, dmd, ndir)
        out = getoutput(cmd)
    except Exception:
        raise
    finally:
        os.chdir(cwd)
    return out


def _parse_output(text):
    """ 
    Parse the output of the NE2001 program into a dictionary.
    """
    lines = [line.strip() for line in text.strip().splitlines()]
    lines = lines[6:] # Ignore comment lines and summary of inputs
    items = {}

    # Check for leading '>' character on first output line when NE2001 has
    # been called to convert DM to DIST. If '>' is there it means that the
    # output distance is only a lower limit
    items['LOWERLIM'] = False
    if lines[0].startswith('>') and 'DIST' in lines[0]:
        items['LOWERLIM'] = True
        lines[0] = lines[0].replace('>', ' ')        

    for line in lines:
        if not line.startswith("#"):
            lis = line.split()
            key = lis[1]
            val = float(lis[0])
            items[key] = val
    return items


def get_output(glon, glat, dmd, ndir):
    """ 
    Run the NE2001 program and parse its output into a dictionary.

    Parameters
    ----------
    glon: float
        Galactic longitude in degrees
    glat: float
        Galactic latitude in degrees
    dmd: float
        DM (pc cm^-3) or Distance (kpc) depending on the direction of conversion
        (see ndir parameter below)
    ndir: int
        If 1, convert DM to distance. If -1, convert distance to DM.

    Returns
    -------
    items: dict
        Dictionary with all output values. The keys are parsed from
        the second output column of the NE2001 program. 

    Raises
    ------
    ValueError:
        If dmd is negative or if ndir is not either -1 or 1
    RuntimeError: 
        If the output of the program cannot be parsed. This usually happens if
        the NE2001 programs returns an error message. In this case, the
        original text output of the program is returned as the message of the
        RuntimeError.
    """
    text = _run_ne2001(glon, glat, dmd, ndir)
    exc = None
    try:
        return _parse_output(text)
    except Exception as err:
        exc = err

    if exc:
        msg = "Failed to parse NE2001 output:\n{!s}".format(text)
        raise RuntimeError(msg)


def get_dm_full(glon, glat, dist_kpc):
    """ 
    Get the NE2001 model DM for given coordinates and distance, and return the
    full output from the program as a dictionary.

    Parameters
    ----------
    glon: float
        Galactic longitude in degrees
    glat: float
        Galactic latitude in degrees
    dist_kpc: float
        Distance in kpc

    Returns
    -------
    items: dict
        Dictionary with all the outputs of the NE2001 program
    """
    return get_output(glon, glat, dist_kpc, -1)


def get_dm(glon, glat, dist_kpc):
    """ 
    Get the NE2001 model DM for given coordinates and distance.

    Parameters
    ----------
    glon: float
        Galactic longitude in degrees
    glat: float
        Galactic latitude in degrees
    dist_kpc: float
        Distance in kpc

    Returns
    -------
    dm: float
        NE2001 model DM in pc cm^-3
    """
    return get_dm_full(glon, glat, dist_kpc)['DM']


def get_dist_full(glon, glat, dm):
    """ Get the NE2001 model distance for given coordinates and DM, and return
    the full output from the program as a dictionary.

    Parameters
    ----------
    glon: float
        Galactic longitude in degrees
    glat: float
        Galactic latitude in degrees
    dm: float
        DM in pc cm^-3

    Returns
    -------
    items: dict
        Dictionary with all the outputs of the NE2001 program
    """
    return get_output(glon, glat, dm, 1)


def get_dist(glon, glat, dm):
    """ Get the NE2001 model distance for given coordinates and DM.

    Parameters
    ----------
    glon: float
        Galactic longitude in degrees
    glat: float
        Galactic latitude in degrees
    dm: float
        DM in pc cm^-3

    Returns
    -------
    dist: float
        NE2001 model distance in kpc
    lower_limit: bool
        True if the distance returned is only a lower limit (input DM exceeds
        Galactic contribution), False otherwise.
    """
    items = get_dist_full(glon, glat, dm)
    return items['DIST'], items['LOWERLIM']


def get_galactic_dm(glon, glat):
    """ Get the Galactic DM contribution for given coordinates. This simply
    calls get_dm() with dist_kpc = 30.0.

    Parameters
    ----------
    glon: float
        Galactic longitude in degrees
    glat: float
        Galactic latitude in degrees

    Returns
    -------
    dm: float
        Galactic DM contribution in pc cm^-3
    """
    return get_dm(glon, glat, 30.0)
